#ifndef __LOG_H
#define __LOG_H

#include <stdio.h>
#include <stdint.h>
#include <errno.h>

#ifdef LOG_PTHREAD
#include <pthread.h>
#endif

typedef struct log_s {

  struct log_s *parent;

  uint16_t  bufsize;
  char *buf;
  uint8_t  level;
  const char *prefix;
  uint8_t flags;

  FILE *file;
  int  facility;

  #ifdef LOG_PTHREAD
  pthread_mutex_t mtx;
  #endif

} log_t;


log_t *log_create(uint16_t size);
void log_free(log_t *l);

void log_set_prefix(log_t *l, const char *prefix);
const char *log_get_prefix(log_t *l);

uint8_t log_get_level(log_t *l);
void log_set_level(log_t *l, uint8_t level);

int  log_get_facility(log_t *l);
void log_set_facility(log_t *l, int facility);

const char *log_facility2str( int facility );
int log_str2facility( const char *name);

int log_open_parent(log_t *l, log_t *f);
int log_open_stream(log_t *l, FILE *f);
int log_open_file(log_t *l, const char *name);
int log_open_sys(log_t *l, int facility);

int log_open(log_t *l, const char *name, int facility);

void log_close(log_t *d);

void log_print(log_t *, uint8_t mlevel, const char *format, ...);
void log_dump(log_t *, void *, uint32_t size);

#define LOG_LVL_DISABLED   0u

#ifndef LOG_LVL_ERROR
 #define LOG_LVL_ERROR     1u
#endif

#ifndef LOG_LVL_WARN
 #define LOG_LVL_WARN      2u
#endif

#ifndef LOG_LVL_MSG
 #define LOG_LVL_MSG       3u
#endif

#ifndef LOG_LVL_INFO
 #define LOG_LVL_INFO      4u
#endif

#ifndef LOG_LVL_DEBUG
 #define LOG_LVL_DEBUG     5u
#endif

#ifndef LOG_STDLVL
 #define LOG_STDLVL        LOG_LVL_DEBUG+100
#endif

#define LOG_LVL_INVALID 0xFF

#ifndef LOG_LVL_DEBUG_BASE
 #define LOG_LVL_DEBUG_BASE   0
#endif

#define DECLARE_LOG_PTR log_t *LOG_PTR

#ifndef LOG_DISABLED
  #ifdef LOG_PTR
   #define LOGMSG(level, format, ...) \
     do { log_print(LOG_PTR, level, format"\n", ##__VA_ARGS__); } while (0)
  #else
   #define LOGMSG(level, format, ...) \
     do { \
          if (level <= LOG_STDLVL ) \
            fprintf((level == LOG_LVL_ERROR)?stderr:stdout,\
               format"\n", ##__VA_ARGS__); \
     } while(0)
  #endif
#else
   #define LOGMSG(level, format, ...)
#endif
#undef LOG_DISABLED

#define ERRMSG(format, ...) \
   LOGMSG(LOG_LVL_ERROR, "ERR: "format, ##__VA_ARGS__)

#define ERR(format, ...) \
   LOGMSG(LOG_LVL_ERROR, "ERR: %s: "format, __FUNCTION__, ##__VA_ARGS__)

#define ERRCALL(s) \
   ERR("%s: %s", s, strerror(errno))

#define WRN(format, ...) \
   LOGMSG(LOG_LVL_WARN, "WRN: %s: "format, __FUNCTION__, ##__VA_ARGS__)

#define LOG(format, ...) \
  LOGMSG(LOG_LVL_INFO, format, ##__VA_ARGS__)

#define MSG(format, ...) \
  LOGMSG(LOG_LVL_MSG, format, ##__VA_ARGS__)

#ifndef LOG_DEBUG_DISABLED
 #define DBGL(level, format, ...) \
    LOGMSG(level+LOG_LVL_DEBUG+LOG_LVL_DEBUG_BASE, "DBG: %s: "format, __FUNCTION__, ##__VA_ARGS__)
#else
 #define DBGL(level, format, ...)
#endif
#undef LOG_DEBUG_DISABLED

#define DBG(format, ...) DBGL(0, format, ##__VA_ARGS__)

#endif
